/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Database } from '../../mol-data/db';
export declare function ensureAvailable(path: string, url: string, forceDownload?: boolean): Promise<void>;
export declare function ensureDataAvailable(forceDownload?: boolean): Promise<void>;
export declare function readFileAsCollection<S extends Database.Schema>(path: string, schema: S): Promise<import("../../mol-data/db").DatabaseCollection<S>>;
export declare function readCCD(): Promise<import("../../mol-data/db").DatabaseCollection<{
    chem_comp: {
        formula: import("../../mol-data/db/column").Column.Schema.Str;
        formula_weight: import("../../mol-data/db/column").Column.Schema.Float;
        id: import("../../mol-data/db/column").Column.Schema.Str;
        mon_nstd_parent_comp_id: import("../../mol-data/db/column").Column.Schema.List<string>;
        name: import("../../mol-data/db/column").Column.Schema.Str;
        one_letter_code: import("../../mol-data/db/column").Column.Schema.Str;
        three_letter_code: import("../../mol-data/db/column").Column.Schema.Str;
        type: import("../../mol-data/db/column").Column.Schema.Aliased<"D-peptide linking" | "L-peptide linking" | "D-peptide NH3 amino terminus" | "L-peptide NH3 amino terminus" | "D-peptide COOH carboxy terminus" | "L-peptide COOH carboxy terminus" | "DNA linking" | "RNA linking" | "L-RNA linking" | "L-DNA linking" | "DNA OH 5 prime terminus" | "RNA OH 5 prime terminus" | "DNA OH 3 prime terminus" | "RNA OH 3 prime terminus" | "D-saccharide, beta linking" | "D-saccharide, alpha linking" | "L-saccharide, beta linking" | "L-saccharide, alpha linking" | "L-saccharide" | "D-saccharide" | "saccharide" | "non-polymer" | "peptide linking" | "peptide-like" | "L-gamma-peptide, C-delta linking" | "D-gamma-peptide, C-delta linking" | "L-beta-peptide, C-gamma linking" | "D-beta-peptide, C-gamma linking" | "other">;
        pdbx_synonyms: import("../../mol-data/db/column").Column.Schema.List<string>;
        pdbx_type: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_ambiguous_flag: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_replaced_by: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_replaces: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_formal_charge: import("../../mol-data/db/column").Column.Schema.Int;
        pdbx_model_coordinates_details: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_model_coordinates_db_code: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_ideal_coordinates_details: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_ideal_coordinates_missing_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
        pdbx_model_coordinates_missing_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
        pdbx_initial_date: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_modified_date: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_release_status: import("../../mol-data/db/column").Column.Schema.Aliased<"REL" | "HOLD" | "HPUB" | "OBS" | "DEL" | "REF_ONLY">;
        pdbx_processing_site: import("../../mol-data/db/column").Column.Schema.Aliased<"RCSB" | "EBI" | "PDBE" | "PDBJ" | "PDBC">;
    };
    chem_comp_atom: {
        alt_atom_id: import("../../mol-data/db/column").Column.Schema.Str;
        atom_id: import("../../mol-data/db/column").Column.Schema.Str;
        charge: import("../../mol-data/db/column").Column.Schema.Int;
        model_Cartn_x: import("../../mol-data/db/column").Column.Schema.Coordinate;
        model_Cartn_y: import("../../mol-data/db/column").Column.Schema.Coordinate;
        model_Cartn_z: import("../../mol-data/db/column").Column.Schema.Coordinate;
        comp_id: import("../../mol-data/db/column").Column.Schema.Str;
        type_symbol: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_align: import("../../mol-data/db/column").Column.Schema.Int;
        pdbx_ordinal: import("../../mol-data/db/column").Column.Schema.Int;
        pdbx_model_Cartn_x_ideal: import("../../mol-data/db/column").Column.Schema.Coordinate;
        pdbx_model_Cartn_y_ideal: import("../../mol-data/db/column").Column.Schema.Coordinate;
        pdbx_model_Cartn_z_ideal: import("../../mol-data/db/column").Column.Schema.Coordinate;
        pdbx_stereo_config: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "R" | "S">;
        pdbx_aromatic_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
        pdbx_leaving_atom_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
    };
    chem_comp_bond: {
        atom_id_1: import("../../mol-data/db/column").Column.Schema.Str;
        atom_id_2: import("../../mol-data/db/column").Column.Schema.Str;
        comp_id: import("../../mol-data/db/column").Column.Schema.Str;
        value_order: import("../../mol-data/db/column").Column.Schema.Aliased<"sing" | "doub" | "trip" | "quad" | "arom" | "poly" | "delo" | "pi">;
        pdbx_ordinal: import("../../mol-data/db/column").Column.Schema.Int;
        pdbx_stereo_config: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "E" | "Z">;
        pdbx_aromatic_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
    };
    pdbx_chem_comp_descriptor: {
        comp_id: import("../../mol-data/db/column").Column.Schema.Str;
        descriptor: import("../../mol-data/db/column").Column.Schema.Str;
        type: import("../../mol-data/db/column").Column.Schema.Aliased<"SMILES_CANNONICAL" | "SMILES_CANONICAL" | "SMILES" | "InChI" | "InChI_MAIN" | "InChI_MAIN_FORMULA" | "InChI_MAIN_CONNECT" | "InChI_MAIN_HATOM" | "InChI_CHARGE" | "InChI_STEREO" | "InChI_ISOTOPE" | "InChI_FIXEDH" | "InChI_RECONNECT" | "InChIKey">;
        program: import("../../mol-data/db/column").Column.Schema.Str;
        program_version: import("../../mol-data/db/column").Column.Schema.Str;
    };
    pdbx_chem_comp_identifier: {
        comp_id: import("../../mol-data/db/column").Column.Schema.Str;
        identifier: import("../../mol-data/db/column").Column.Schema.Str;
        type: import("../../mol-data/db/column").Column.Schema.Aliased<"COMMON NAME" | "SYSTEMATIC NAME" | "CAS REGISTRY NUMBER" | "PUBCHEM Identifier" | "MDL Identifier" | "SYNONYM" | "CONDENSED IUPAC CARB SYMBOL" | "IUPAC CARB SYMBOL" | "SNFG CARB SYMBOL" | "CONDENSED IUPAC CARBOHYDRATE SYMBOL" | "IUPAC CARBOHYDRATE SYMBOL" | "SNFG CARBOHYDRATE SYMBOL">;
        program: import("../../mol-data/db/column").Column.Schema.Str;
        program_version: import("../../mol-data/db/column").Column.Schema.Str;
    };
}>>;
export declare function readPVCD(): Promise<import("../../mol-data/db").DatabaseCollection<{
    chem_comp: {
        formula: import("../../mol-data/db/column").Column.Schema.Str;
        formula_weight: import("../../mol-data/db/column").Column.Schema.Float;
        id: import("../../mol-data/db/column").Column.Schema.Str;
        mon_nstd_parent_comp_id: import("../../mol-data/db/column").Column.Schema.List<string>;
        name: import("../../mol-data/db/column").Column.Schema.Str;
        one_letter_code: import("../../mol-data/db/column").Column.Schema.Str;
        three_letter_code: import("../../mol-data/db/column").Column.Schema.Str;
        type: import("../../mol-data/db/column").Column.Schema.Aliased<"D-peptide linking" | "L-peptide linking" | "D-peptide NH3 amino terminus" | "L-peptide NH3 amino terminus" | "D-peptide COOH carboxy terminus" | "L-peptide COOH carboxy terminus" | "DNA linking" | "RNA linking" | "L-RNA linking" | "L-DNA linking" | "DNA OH 5 prime terminus" | "RNA OH 5 prime terminus" | "DNA OH 3 prime terminus" | "RNA OH 3 prime terminus" | "D-saccharide, beta linking" | "D-saccharide, alpha linking" | "L-saccharide, beta linking" | "L-saccharide, alpha linking" | "L-saccharide" | "D-saccharide" | "saccharide" | "non-polymer" | "peptide linking" | "peptide-like" | "L-gamma-peptide, C-delta linking" | "D-gamma-peptide, C-delta linking" | "L-beta-peptide, C-gamma linking" | "D-beta-peptide, C-gamma linking" | "other">;
        pdbx_synonyms: import("../../mol-data/db/column").Column.Schema.List<string>;
        pdbx_type: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_ambiguous_flag: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_replaced_by: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_replaces: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_formal_charge: import("../../mol-data/db/column").Column.Schema.Int;
        pdbx_model_coordinates_details: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_model_coordinates_db_code: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_ideal_coordinates_details: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_ideal_coordinates_missing_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
        pdbx_model_coordinates_missing_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
        pdbx_initial_date: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_modified_date: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_release_status: import("../../mol-data/db/column").Column.Schema.Aliased<"REL" | "HOLD" | "HPUB" | "OBS" | "DEL" | "REF_ONLY">;
        pdbx_processing_site: import("../../mol-data/db/column").Column.Schema.Aliased<"RCSB" | "EBI" | "PDBE" | "PDBJ" | "PDBC">;
    };
    chem_comp_atom: {
        alt_atom_id: import("../../mol-data/db/column").Column.Schema.Str;
        atom_id: import("../../mol-data/db/column").Column.Schema.Str;
        charge: import("../../mol-data/db/column").Column.Schema.Int;
        model_Cartn_x: import("../../mol-data/db/column").Column.Schema.Coordinate;
        model_Cartn_y: import("../../mol-data/db/column").Column.Schema.Coordinate;
        model_Cartn_z: import("../../mol-data/db/column").Column.Schema.Coordinate;
        comp_id: import("../../mol-data/db/column").Column.Schema.Str;
        type_symbol: import("../../mol-data/db/column").Column.Schema.Str;
        pdbx_align: import("../../mol-data/db/column").Column.Schema.Int;
        pdbx_ordinal: import("../../mol-data/db/column").Column.Schema.Int;
        pdbx_model_Cartn_x_ideal: import("../../mol-data/db/column").Column.Schema.Coordinate;
        pdbx_model_Cartn_y_ideal: import("../../mol-data/db/column").Column.Schema.Coordinate;
        pdbx_model_Cartn_z_ideal: import("../../mol-data/db/column").Column.Schema.Coordinate;
        pdbx_stereo_config: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "R" | "S">;
        pdbx_aromatic_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
        pdbx_leaving_atom_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
    };
    chem_comp_bond: {
        atom_id_1: import("../../mol-data/db/column").Column.Schema.Str;
        atom_id_2: import("../../mol-data/db/column").Column.Schema.Str;
        comp_id: import("../../mol-data/db/column").Column.Schema.Str;
        value_order: import("../../mol-data/db/column").Column.Schema.Aliased<"sing" | "doub" | "trip" | "quad" | "arom" | "poly" | "delo" | "pi">;
        pdbx_ordinal: import("../../mol-data/db/column").Column.Schema.Int;
        pdbx_stereo_config: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "E" | "Z">;
        pdbx_aromatic_flag: import("../../mol-data/db/column").Column.Schema.Aliased<"N" | "Y">;
    };
    pdbx_chem_comp_descriptor: {
        comp_id: import("../../mol-data/db/column").Column.Schema.Str;
        descriptor: import("../../mol-data/db/column").Column.Schema.Str;
        type: import("../../mol-data/db/column").Column.Schema.Aliased<"SMILES_CANNONICAL" | "SMILES_CANONICAL" | "SMILES" | "InChI" | "InChI_MAIN" | "InChI_MAIN_FORMULA" | "InChI_MAIN_CONNECT" | "InChI_MAIN_HATOM" | "InChI_CHARGE" | "InChI_STEREO" | "InChI_ISOTOPE" | "InChI_FIXEDH" | "InChI_RECONNECT" | "InChIKey">;
        program: import("../../mol-data/db/column").Column.Schema.Str;
        program_version: import("../../mol-data/db/column").Column.Schema.Str;
    };
    pdbx_chem_comp_identifier: {
        comp_id: import("../../mol-data/db/column").Column.Schema.Str;
        identifier: import("../../mol-data/db/column").Column.Schema.Str;
        type: import("../../mol-data/db/column").Column.Schema.Aliased<"COMMON NAME" | "SYSTEMATIC NAME" | "CAS REGISTRY NUMBER" | "PUBCHEM Identifier" | "MDL Identifier" | "SYNONYM" | "CONDENSED IUPAC CARB SYMBOL" | "IUPAC CARB SYMBOL" | "SNFG CARB SYMBOL" | "CONDENSED IUPAC CARBOHYDRATE SYMBOL" | "IUPAC CARBOHYDRATE SYMBOL" | "SNFG CARBOHYDRATE SYMBOL">;
        program: import("../../mol-data/db/column").Column.Schema.Str;
        program_version: import("../../mol-data/db/column").Column.Schema.Str;
    };
}>>;
export declare function getEncodedCif(name: string, database: Database<Database.Schema>, binary?: boolean): string | Uint8Array;
