"use strict";
/**
 * Copyright (c) 2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateOrbitalVolume = exports.StaticBasisAndOrbitals = exports.BasisAndOrbitals = void 0;
var tslib_1 = require("tslib");
var objects_1 = require("../../mol-plugin-state/objects");
var cubes_1 = require("../../extensions/alpha-orbitals/cubes");
var param_definition_1 = require("../../mol-util/param-definition");
var mol_task_1 = require("../../mol-task");
var custom_property_1 = require("../../mol-model/custom-property");
var BasisAndOrbitals = /** @class */ (function (_super) {
    tslib_1.__extends(BasisAndOrbitals, _super);
    function BasisAndOrbitals() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return BasisAndOrbitals;
}(objects_1.PluginStateObject.Create({ name: 'Basis', typeClass: 'Object' })));
exports.BasisAndOrbitals = BasisAndOrbitals;
exports.StaticBasisAndOrbitals = objects_1.PluginStateTransform.BuiltIn({
    name: 'static-basis-and-orbitals',
    display: 'Basis and Orbitals',
    from: objects_1.PluginStateObject.Root,
    to: BasisAndOrbitals,
    params: {
        label: param_definition_1.ParamDefinition.Text('Orbital Data', { isHidden: true }),
        basis: param_definition_1.ParamDefinition.Value(void 0, { isHidden: true }),
        order: param_definition_1.ParamDefinition.Text('gaussian', { isHidden: true }),
        orbitals: param_definition_1.ParamDefinition.Value([], { isHidden: true })
    },
})({
    apply: function (_a) {
        var params = _a.params;
        return new BasisAndOrbitals({ basis: params.basis, order: params.order, orbitals: params.orbitals }, { label: params.label });
    }
});
exports.CreateOrbitalVolume = objects_1.PluginStateTransform.BuiltIn({
    name: 'create-orbital-volume',
    display: 'Orbital Volume',
    from: BasisAndOrbitals,
    to: objects_1.PluginStateObject.Volume.Data,
    params: function (a) {
        if (!a) {
            return { index: param_definition_1.ParamDefinition.Numeric(0) };
        }
        return {
            index: param_definition_1.ParamDefinition.Select(0, a.data.orbitals.map(function (o, i) { return [i, "[" + (i + 1) + "] " + o.energy.toFixed(4)]; }))
        };
    }
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Orbital Volume', function (ctx) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var data, volume;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, cubes_1.createSphericalCollocationGrid({
                            basis: a.data.basis,
                            cutoffThreshold: 0.0075,
                            alphaOrbitals: a.data.orbitals[params.index].alpha,
                            sphericalOrder: a.data.order,
                            boxExpand: 4.5,
                            gridSpacing: [
                                [55, 0.6],
                                [40, 0.5],
                                [25, 0.4],
                                [0, 0.33],
                            ],
                        }).runInContext(ctx)];
                    case 1:
                        data = _a.sent();
                        volume = {
                            grid: data.grid,
                            sourceData: { name: 'custom grid', kind: 'custom', data: data },
                            customProperties: new custom_property_1.CustomProperties(),
                            _propertyData: Object.create(null),
                        };
                        return [2 /*return*/, new objects_1.PluginStateObject.Volume.Data(volume, { label: 'Orbital Volume' })];
                }
            });
        }); });
    }
});
//# sourceMappingURL=transforms.js.map