"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse3DG = exports.Schema3DG = void 0;
var tslib_1 = require("tslib");
var result_1 = require("../result");
var mol_task_1 = require("../../../mol-task");
var parser_1 = require("../csv/parser");
var db_1 = require("../../../mol-data/db");
var schema_1 = require("../cif/schema");
var Schema = db_1.Column.Schema;
exports.Schema3DG = {
    /** Chromosome name */
    chromosome: Schema.str,
    /** Base position */
    position: Schema.int,
    /** X coordinate */
    x: Schema.float,
    /** Y coordinate */
    y: Schema.float,
    /** Z coordinate */
    z: Schema.float,
};
var FieldNames = ['chromosome', 'position', 'x', 'y', 'z'];
function categoryFromTable(name, table) {
    return {
        name: name,
        rowCount: table.rowCount,
        fieldNames: FieldNames,
        getField: function (name) {
            return table.getColumn(FieldNames.indexOf(name).toString());
        }
    };
}
function parse3DG(data) {
    var _this = this;
    return mol_task_1.Task.create('Parse 3DG', function (ctx) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var opts, csvFile, category, table;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    opts = { quote: '', comment: '#', delimiter: '\t', noColumnNames: true };
                    return [4 /*yield*/, parser_1.parseCsv(data, opts).runInContext(ctx)];
                case 1:
                    csvFile = _a.sent();
                    if (csvFile.isError)
                        return [2 /*return*/, result_1.ReaderResult.error(csvFile.message, csvFile.line)];
                    category = categoryFromTable('3dg', csvFile.result.table);
                    table = schema_1.toTable(exports.Schema3DG, category);
                    return [2 /*return*/, result_1.ReaderResult.success({ table: table })];
            }
        });
    }); });
}
exports.parse3DG = parse3DG;
//# sourceMappingURL=parser.js.map