/**
 * Copyright (c) 2019-2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { RepresentationParamsGetter, RepresentationContext } from '../../../mol-repr/representation';
import { ThemeRegistryContext } from '../../../mol-theme/theme';
import { Structure } from '../../../mol-model/structure';
import { StructureRepresentation, StructureRepresentationProvider } from '../../../mol-repr/structure/representation';
export declare const EllipsoidParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic">;
    sizeFactor: PD.Numeric;
    sizeAspectRatio: PD.Numeric;
    linkCap: PD.BooleanParam;
    visuals: PD.MultiSelect<"intra-bond" | "inter-bond" | "ellipsoid-mesh">;
    tryUseImpostor: PD.BooleanParam;
    includeTypes: PD.MultiSelect<"covalent" | "metal-coordination" | "hydrogen-bond" | "disulfide" | "aromatic" | "computed">;
    excludeTypes: PD.MultiSelect<"covalent" | "metal-coordination" | "hydrogen-bond" | "disulfide" | "aromatic" | "computed">;
    ignoreHydrogens: PD.BooleanParam;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    dashCount: PD.Numeric;
    dashScale: PD.Numeric;
    dashCap: PD.BooleanParam;
    radialSegments: PD.Numeric;
    doubleSided: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    detail: PD.Numeric;
};
export declare type EllipsoidParams = typeof EllipsoidParams;
export declare function getEllipsoidParams(ctx: ThemeRegistryContext, structure: Structure): {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic">;
    sizeFactor: PD.Numeric;
    sizeAspectRatio: PD.Numeric;
    linkCap: PD.BooleanParam;
    visuals: PD.MultiSelect<"intra-bond" | "inter-bond" | "ellipsoid-mesh">;
    tryUseImpostor: PD.BooleanParam;
    includeTypes: PD.MultiSelect<"covalent" | "metal-coordination" | "hydrogen-bond" | "disulfide" | "aromatic" | "computed">;
    excludeTypes: PD.MultiSelect<"covalent" | "metal-coordination" | "hydrogen-bond" | "disulfide" | "aromatic" | "computed">;
    ignoreHydrogens: PD.BooleanParam;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    dashCount: PD.Numeric;
    dashScale: PD.Numeric;
    dashCap: PD.BooleanParam;
    radialSegments: PD.Numeric;
    doubleSided: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    detail: PD.Numeric;
};
export declare type EllipsoidRepresentation = StructureRepresentation<EllipsoidParams>;
export declare function EllipsoidRepresentation(ctx: RepresentationContext, getParams: RepresentationParamsGetter<Structure, EllipsoidParams>): EllipsoidRepresentation;
export declare const EllipsoidRepresentationProvider: StructureRepresentationProvider<{
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic">;
    sizeFactor: PD.Numeric;
    sizeAspectRatio: PD.Numeric;
    linkCap: PD.BooleanParam;
    visuals: PD.MultiSelect<"intra-bond" | "inter-bond" | "ellipsoid-mesh">;
    tryUseImpostor: PD.BooleanParam;
    includeTypes: PD.MultiSelect<"covalent" | "metal-coordination" | "hydrogen-bond" | "disulfide" | "aromatic" | "computed">;
    excludeTypes: PD.MultiSelect<"covalent" | "metal-coordination" | "hydrogen-bond" | "disulfide" | "aromatic" | "computed">;
    ignoreHydrogens: PD.BooleanParam;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    dashCount: PD.Numeric;
    dashScale: PD.Numeric;
    dashCap: PD.BooleanParam;
    radialSegments: PD.Numeric;
    doubleSided: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    detail: PD.Numeric;
}, "ellipsoid">;
