"use strict";
/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnitsTextureMeshVisual = exports.UnitsTextureMeshParams = exports.UnitsDirectVolumeVisual = exports.UnitsDirectVolumeParams = exports.UnitsTextVisual = exports.UnitsTextParams = exports.UnitsLinesVisual = exports.UnitsLinesParams = exports.UnitsPointsVisual = exports.UnitsPointsParams = exports.UnitsCylindersVisual = exports.UnitsCylindersParams = exports.UnitsSpheresVisual = exports.UnitsSpheresParams = exports.UnitsMeshVisual = exports.UnitsMeshParams = exports.UnitsVisual = void 0;
var tslib_1 = require("tslib");
var structure_1 = require("../../mol-model/structure");
var visual_1 = require("../visual");
var geometry_1 = require("../../mol-geo/geometry/geometry");
var theme_1 = require("../../mol-theme/theme");
var common_1 = require("./visual/util/common");
var render_object_1 = require("../../mol-gl/render-object");
var loci_1 = require("../../mol-model/loci");
var int_1 = require("../../mol-data/int");
var util_1 = require("../util");
var color_1 = require("../../mol-theme/color");
var marker_data_1 = require("../../mol-geo/geometry/marker-data");
var mol_util_1 = require("../../mol-util");
var size_data_1 = require("../../mol-geo/geometry/size-data");
var color_data_1 = require("../../mol-geo/geometry/color-data");
var mesh_1 = require("../../mol-geo/geometry/mesh/mesh");
var size_1 = require("../../mol-theme/size");
var spheres_1 = require("../../mol-geo/geometry/spheres/spheres");
var cylinders_1 = require("../../mol-geo/geometry/cylinders/cylinders");
var points_1 = require("../../mol-geo/geometry/points/points");
var lines_1 = require("../../mol-geo/geometry/lines/lines");
var text_1 = require("../../mol-geo/geometry/text/text");
var direct_volume_1 = require("../../mol-geo/geometry/direct-volume/direct-volume");
var texture_mesh_1 = require("../../mol-geo/geometry/texture-mesh/texture-mesh");
var params_1 = require("./params");
function createUnitsRenderObject(group, geometry, locationIt, theme, props, materialId) {
    var _a = geometry_1.Geometry.getUtils(geometry), createValues = _a.createValues, createRenderableState = _a.createRenderableState;
    var transform = common_1.createUnitsTransform(group);
    var values = createValues(geometry, transform, locationIt, theme, props);
    var state = createRenderableState(props);
    return render_object_1.createRenderObject(geometry.kind, values, state, materialId);
}
function UnitsVisual(builder, materialId) {
    var defaultProps = builder.defaultProps, createGeometry = builder.createGeometry, createLocationIterator = builder.createLocationIterator, getLoci = builder.getLoci, eachLocation = builder.eachLocation, setUpdateState = builder.setUpdateState, mustRecreate = builder.mustRecreate, dispose = builder.dispose;
    var _a = builder.geometryUtils, createEmptyGeometry = _a.createEmpty, updateValues = _a.updateValues, updateBoundingSphere = _a.updateBoundingSphere, updateRenderableState = _a.updateRenderableState, createPositionIterator = _a.createPositionIterator;
    var updateState = util_1.VisualUpdateState.create();
    var renderObject;
    var newProps = Object.assign({}, defaultProps);
    var newTheme = theme_1.Theme.createEmpty();
    var newStructureGroup;
    var currentProps;
    var currentTheme;
    var currentStructureGroup;
    var geometry;
    var locationIt;
    var positionIt;
    function prepareUpdate(theme, props, structureGroup) {
        if (!structureGroup && !currentStructureGroup) {
            throw new Error('missing structureGroup');
        }
        newProps = props;
        newTheme = theme;
        newStructureGroup = structureGroup;
        util_1.VisualUpdateState.reset(updateState);
        if (!renderObject || !currentStructureGroup) {
            // console.log('create new');
            updateState.createNew = true;
            updateState.createGeometry = true;
            return;
        }
        setUpdateState(updateState, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
        if (!structure_1.Structure.areHierarchiesEqual(currentStructureGroup.structure, newStructureGroup.structure)) {
            // console.log('new hierarchy');
            updateState.updateTransform = true;
            updateState.updateColor = true;
            updateState.updateSize = true;
        }
        if (!color_1.ColorTheme.areEqual(newTheme.color, currentTheme.color)) {
            // console.log('new colorTheme');
            updateState.updateColor = true;
        }
        if (!mol_util_1.deepEqual(newProps.unitKinds, currentProps.unitKinds)) {
            // console.log('new unitKinds');
            updateState.createGeometry = true;
        }
        if (newStructureGroup.group.transformHash !== currentStructureGroup.group.transformHash) {
            // console.log('new transformHash');
            if (newStructureGroup.group.units.length !== currentStructureGroup.group.units.length || updateState.updateColor) {
                updateState.updateTransform = true;
            }
            else {
                updateState.updateMatrix = true;
            }
        }
        // check if the operator or conformation of unit has changed
        var newUnit = newStructureGroup.group.units[0];
        var currentUnit = currentStructureGroup.group.units[0];
        if (!structure_1.Unit.areOperatorsEqual(newUnit, currentUnit)) {
            // console.log('new operators');
            updateState.updateTransform = true;
        }
        if (!structure_1.Unit.areConformationsEqual(newUnit, currentUnit)) {
            // console.log('new conformation');
            updateState.createGeometry = true;
        }
        if (updateState.updateTransform) {
            updateState.updateMatrix = true;
        }
        if (updateState.updateSize && !('uSize' in renderObject.values)) {
            updateState.createGeometry = true;
        }
        if (updateState.createGeometry || updateState.updateTransform) {
            if (currentStructureGroup.structure.hashCode !== newStructureGroup.structure.hashCode) {
                // console.log('new hashCode');
                updateState.updateColor = true;
                updateState.updateSize = true;
            }
            if (newTheme.color.granularity.startsWith('vertex')) {
                updateState.updateColor = true;
            }
        }
    }
    function update(newGeometry) {
        if (updateState.createNew) {
            locationIt = createLocationIterator(newStructureGroup);
            if (newGeometry) {
                renderObject = createUnitsRenderObject(newStructureGroup.group, newGeometry, locationIt, newTheme, newProps, materialId);
                positionIt = createPositionIterator(newGeometry, renderObject.values);
            }
            else {
                throw new Error('expected geometry to be given');
            }
        }
        else {
            if (!renderObject) {
                throw new Error('expected renderObject to be available');
            }
            if (updateState.updateTransform) {
                // console.log('update transform');
                locationIt = createLocationIterator(newStructureGroup);
                var instanceCount = locationIt.instanceCount, groupCount = locationIt.groupCount;
                marker_data_1.createMarkers(instanceCount * groupCount, renderObject.values);
            }
            if (updateState.updateMatrix) {
                // console.log('update matrix');
                common_1.createUnitsTransform(newStructureGroup.group, renderObject.values);
            }
            if (updateState.createGeometry) {
                // console.log('update geometry');
                if (newGeometry) {
                    mol_util_1.ValueCell.updateIfChanged(renderObject.values.drawCount, geometry_1.Geometry.getDrawCount(newGeometry));
                    mol_util_1.ValueCell.updateIfChanged(renderObject.values.uVertexCount, geometry_1.Geometry.getVertexCount(newGeometry));
                }
                else {
                    throw new Error('expected geometry to be given');
                }
            }
            if (updateState.updateTransform || updateState.createGeometry) {
                updateBoundingSphere(renderObject.values, newGeometry || geometry);
                positionIt = createPositionIterator(newGeometry || geometry, renderObject.values);
            }
            if (updateState.updateSize) {
                // not all geometries have size data, so check here
                if ('uSize' in renderObject.values) {
                    // console.log('update size');
                    size_data_1.createSizes(locationIt, newTheme.size, renderObject.values);
                }
            }
            if (updateState.updateColor) {
                // console.log('update color');
                color_data_1.createColors(locationIt, positionIt, newTheme.color, renderObject.values);
            }
            updateValues(renderObject.values, newProps);
            updateRenderableState(renderObject.state, newProps);
        }
        currentProps = newProps;
        currentTheme = newTheme;
        currentStructureGroup = newStructureGroup;
        if (newGeometry)
            geometry = newGeometry;
    }
    function _createGeometry(ctx, unit, structure, theme, props, geometry) {
        return common_1.includesUnitKind(props.unitKinds, unit)
            ? createGeometry(ctx, unit, structure, theme, props, geometry)
            : createEmptyGeometry(geometry);
    }
    function lociIsSuperset(loci) {
        if (loci_1.isEveryLoci(loci))
            return true;
        if (structure_1.Structure.isLoci(loci) && structure_1.Structure.areRootsEquivalent(loci.structure, currentStructureGroup.structure))
            return true;
        if (structure_1.StructureElement.Loci.is(loci) && structure_1.Structure.areRootsEquivalent(loci.structure, currentStructureGroup.structure)) {
            if (structure_1.StructureElement.Loci.isWholeStructure(loci))
                return true;
        }
        return false;
    }
    function lociApply(loci, apply, isMarking) {
        if (lociIsSuperset(loci)) {
            return apply(int_1.Interval.ofBounds(0, locationIt.groupCount * locationIt.instanceCount));
        }
        else {
            return eachLocation(loci, currentStructureGroup, apply, isMarking);
        }
    }
    return {
        get groupCount() { return locationIt ? locationIt.count : 0; },
        get renderObject() { return locationIt && locationIt.count ? renderObject : undefined; },
        createOrUpdate: function (ctx, theme, props, structureGroup) {
            prepareUpdate(theme, props, structureGroup || currentStructureGroup);
            if (updateState.createGeometry) {
                var newGeometry = _createGeometry(ctx, newStructureGroup.group.units[0], newStructureGroup.structure, newTheme, newProps, geometry);
                return newGeometry instanceof Promise ? newGeometry.then(update) : update(newGeometry);
            }
            else {
                update();
            }
        },
        getLoci: function (pickingId) {
            return renderObject ? getLoci(pickingId, currentStructureGroup, renderObject.id) : loci_1.EmptyLoci;
        },
        mark: function (loci, action) {
            return visual_1.Visual.mark(renderObject, loci, action, lociApply);
        },
        setVisibility: function (visible) {
            visual_1.Visual.setVisibility(renderObject, visible);
        },
        setAlphaFactor: function (alphaFactor) {
            visual_1.Visual.setAlphaFactor(renderObject, alphaFactor);
        },
        setPickable: function (pickable) {
            visual_1.Visual.setPickable(renderObject, pickable);
        },
        setColorOnly: function (colorOnly) {
            visual_1.Visual.setColorOnly(renderObject, colorOnly);
        },
        setTransform: function (matrix, instanceMatrices) {
            visual_1.Visual.setTransform(renderObject, matrix, instanceMatrices);
        },
        setOverpaint: function (overpaint) {
            visual_1.Visual.setOverpaint(renderObject, overpaint, lociApply, true);
        },
        setTransparency: function (transparency) {
            visual_1.Visual.setTransparency(renderObject, transparency, lociApply, true);
        },
        setClipping: function (clipping) {
            visual_1.Visual.setClipping(renderObject, clipping, lociApply, true);
        },
        destroy: function () {
            dispose === null || dispose === void 0 ? void 0 : dispose(geometry);
            if (renderObject) {
                renderObject.state.disposed = true;
                renderObject = undefined;
            }
        },
        mustRecreate: mustRecreate
    };
}
exports.UnitsVisual = UnitsVisual;
// mesh
exports.UnitsMeshParams = tslib_1.__assign(tslib_1.__assign({}, params_1.StructureMeshParams), params_1.StructureParams);
function UnitsMeshVisual(builder, materialId) {
    return UnitsVisual(tslib_1.__assign(tslib_1.__assign({}, builder), { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.createGeometry = true;
        }, geometryUtils: mesh_1.Mesh.Utils }), materialId);
}
exports.UnitsMeshVisual = UnitsMeshVisual;
// spheres
exports.UnitsSpheresParams = tslib_1.__assign(tslib_1.__assign({}, params_1.StructureSpheresParams), params_1.StructureParams);
function UnitsSpheresVisual(builder, materialId) {
    return UnitsVisual(tslib_1.__assign(tslib_1.__assign({}, builder), { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.updateSize = true;
        }, geometryUtils: spheres_1.Spheres.Utils }), materialId);
}
exports.UnitsSpheresVisual = UnitsSpheresVisual;
// cylinders
exports.UnitsCylindersParams = tslib_1.__assign(tslib_1.__assign({}, params_1.StructureCylindersParams), params_1.StructureParams);
function UnitsCylindersVisual(builder, materialId) {
    return UnitsVisual(tslib_1.__assign(tslib_1.__assign({}, builder), { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.updateSize = true;
        }, geometryUtils: cylinders_1.Cylinders.Utils }), materialId);
}
exports.UnitsCylindersVisual = UnitsCylindersVisual;
// points
exports.UnitsPointsParams = tslib_1.__assign(tslib_1.__assign({}, params_1.StructurePointsParams), params_1.StructureParams);
function UnitsPointsVisual(builder, materialId) {
    return UnitsVisual(tslib_1.__assign(tslib_1.__assign({}, builder), { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.updateSize = true;
        }, geometryUtils: points_1.Points.Utils }), materialId);
}
exports.UnitsPointsVisual = UnitsPointsVisual;
// lines
exports.UnitsLinesParams = tslib_1.__assign(tslib_1.__assign({}, params_1.StructureLinesParams), params_1.StructureParams);
function UnitsLinesVisual(builder, materialId) {
    return UnitsVisual(tslib_1.__assign(tslib_1.__assign({}, builder), { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.updateSize = true;
        }, geometryUtils: lines_1.Lines.Utils }), materialId);
}
exports.UnitsLinesVisual = UnitsLinesVisual;
// text
exports.UnitsTextParams = tslib_1.__assign(tslib_1.__assign({}, params_1.StructureTextParams), params_1.StructureParams);
function UnitsTextVisual(builder, materialId) {
    return UnitsVisual(tslib_1.__assign(tslib_1.__assign({}, builder), { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.updateSize = true;
            if (newProps.background !== currentProps.background)
                state.createGeometry = true;
            if (newProps.backgroundMargin !== currentProps.backgroundMargin)
                state.createGeometry = true;
            if (newProps.tether !== currentProps.tether)
                state.createGeometry = true;
            if (newProps.tetherLength !== currentProps.tetherLength)
                state.createGeometry = true;
            if (newProps.tetherBaseWidth !== currentProps.tetherBaseWidth)
                state.createGeometry = true;
            if (newProps.attachment !== currentProps.attachment)
                state.createGeometry = true;
            if (newProps.fontFamily !== currentProps.fontFamily)
                state.createGeometry = true;
            if (newProps.fontQuality !== currentProps.fontQuality)
                state.createGeometry = true;
            if (newProps.fontStyle !== currentProps.fontStyle)
                state.createGeometry = true;
            if (newProps.fontVariant !== currentProps.fontVariant)
                state.createGeometry = true;
            if (newProps.fontWeight !== currentProps.fontWeight)
                state.createGeometry = true;
        }, geometryUtils: text_1.Text.Utils }), materialId);
}
exports.UnitsTextVisual = UnitsTextVisual;
// direct-volume
exports.UnitsDirectVolumeParams = tslib_1.__assign(tslib_1.__assign({}, params_1.StructureDirectVolumeParams), params_1.StructureParams);
function UnitsDirectVolumeVisual(builder, materialId) {
    return UnitsVisual(tslib_1.__assign(tslib_1.__assign({}, builder), { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.createGeometry = true;
        }, geometryUtils: direct_volume_1.DirectVolume.Utils }), materialId);
}
exports.UnitsDirectVolumeVisual = UnitsDirectVolumeVisual;
// texture-mesh
exports.UnitsTextureMeshParams = tslib_1.__assign(tslib_1.__assign({}, params_1.StructureTextureMeshParams), params_1.StructureParams);
function UnitsTextureMeshVisual(builder, materialId) {
    return UnitsVisual(tslib_1.__assign(tslib_1.__assign({}, builder), { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.createGeometry = true;
        }, geometryUtils: texture_mesh_1.TextureMesh.Utils }), materialId);
}
exports.UnitsTextureMeshVisual = UnitsTextureMeshVisual;
//# sourceMappingURL=units-visual.js.map