/**
 * Copyright (c) 2018-2019 Mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Type } from '../type';
import * as Core from './core';
import { Arguments, Argument } from '../symbol';
export declare namespace Types {
    const ElementSymbol: Type.Value<unknown>;
    const AtomName: Type.Value<unknown>;
    const BondFlag: Type.OneOf<string>;
    const BondFlags: Type.Container<number>;
    const SecondaryStructureFlag: Type.OneOf<string>;
    const SecondaryStructureFlags: Type.Container<number>;
    const RingFingerprint: Type.Value<unknown>;
    const EntityType: Type.OneOf<string>;
    const EntitySubtype: Type.OneOf<string>;
    const ObjectPrimitive: Type.OneOf<string>;
    const ResidueId: Type.Value<unknown>;
    const ElementSet: Type.Value<unknown>;
    const ElementSelection: Type.Value<unknown>;
    const ElementReference: Type.Value<unknown>;
    const ElementSelectionQuery: Type.Container<(env: any) => unknown>;
}
export declare const structureQuery: {
    '@header': string;
    type: {
        '@header': string;
        elementSymbol: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Value<string>>;
        }>>, Type.Value<unknown>>;
        atomName: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.AnyValue>;
        }>>, Type.Value<unknown>>;
        entityType: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.OneOf<string>>;
        }>>, Type.OneOf<string>>;
        bondFlags: import("../symbol").MSymbol<Arguments<{
            [key: string]: string;
        }>, Type.Container<number>>;
        ringFingerprint: import("../symbol").MSymbol<Arguments<{
            [key: string]: unknown;
        }>, Type.Value<unknown>>;
        secondaryStructureFlags: import("../symbol").MSymbol<Arguments<{
            [key: string]: string;
        }>, Type.Container<number>>;
        authResidueId: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Value<string>>;
            1: Argument<Type.Value<number>>;
            2: Argument<Type.Value<string>>;
        }>>, Type.Value<unknown>>;
        labelResidueId: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Value<string>>;
            1: Argument<Type.Value<string>>;
            2: Argument<Type.Value<number>>;
            3: Argument<Type.Value<string>>;
        }>>, Type.Value<unknown>>;
    };
    slot: {
        '@header': string;
        element: import("../symbol").MSymbol<Arguments<{}>, Type.Value<unknown>>;
        elementSetReduce: import("../symbol").MSymbol<Arguments<{}>, Type.Variable<any>>;
    };
    generator: {
        '@header': string;
        all: import("../symbol").MSymbol<Arguments<{}>, Type.Container<(env: any) => unknown>>;
        atomGroups: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            'entity-test': Argument<Type.OneOf<boolean>>;
            'chain-test': Argument<Type.OneOf<boolean>>;
            'residue-test': Argument<Type.OneOf<boolean>>;
            'atom-test': Argument<Type.OneOf<boolean>>;
            'group-by': Argument<Type.Any>;
        }>>, Type.Container<(env: any) => unknown>>;
        bondedAtomicPairs: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.OneOf<boolean>>;
        }>>, Type.Container<(env: any) => unknown>>;
        rings: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            fingerprint: Argument<Type.Value<unknown>>;
            'only-aromatic': Argument<Type.OneOf<boolean>>;
        }>>, Type.Container<(env: any) => unknown>>;
        queryInSelection: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            query: Argument<Type.Container<(env: any) => unknown>>;
            'in-complement': Argument<Type.OneOf<boolean>>;
        }>>, Type.Container<(env: any) => unknown>>;
        empty: import("../symbol").MSymbol<Arguments<{}>, Type.Container<(env: any) => unknown>>;
    };
    modifier: {
        '@header': string;
        queryEach: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            query: Argument<Type.Container<(env: any) => unknown>>;
        }>>, Type.Container<(env: any) => unknown>>;
        intersectBy: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            by: Argument<Type.Container<(env: any) => unknown>>;
        }>>, Type.Container<(env: any) => unknown>>;
        exceptBy: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            by: Argument<Type.Container<(env: any) => unknown>>;
        }>>, Type.Container<(env: any) => unknown>>;
        unionBy: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            by: Argument<Type.Container<(env: any) => unknown>>;
        }>>, Type.Container<(env: any) => unknown>>;
        union: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
        }>>, Type.Container<(env: any) => unknown>>;
        cluster: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            'min-distance': Argument<Type.Value<number>>;
            'max-distance': Argument<Type.Value<number>>;
            'min-size': Argument<Type.Value<number>>;
            'max-size': Argument<Type.Value<number>>;
        }>>, Type.Container<(env: any) => unknown>>;
        includeSurroundings: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            radius: Argument<Type.Value<number>>;
            'atom-radius': Argument<Type.Value<number>>;
            'as-whole-residues': Argument<Type.OneOf<boolean>>;
        }>>, Type.Container<(env: any) => unknown>>;
        includeConnected: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            'bond-test': Argument<Type.OneOf<boolean>>;
            'layer-count': Argument<Type.Value<number>>;
            'fixed-point': Argument<Type.OneOf<boolean>>;
            'as-whole-residues': Argument<Type.OneOf<boolean>>;
        }>>, Type.Container<(env: any) => unknown>>;
        wholeResidues: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
        }>>, Type.Container<(env: any) => unknown>>;
        expandProperty: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            property: Argument<Type.AnyValue>;
        }>>, Type.Container<(env: any) => unknown>>;
    };
    filter: {
        '@header': string;
        pick: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            test: Argument<Type.OneOf<boolean>>;
        }>>, Type.Container<(env: any) => unknown>>;
        first: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
        }>>, Type.Container<(env: any) => unknown>>;
        withSameAtomProperties: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            source: Argument<Type.Container<(env: any) => unknown>>;
            property: Argument<Type.Any>;
        }>>, Type.Container<(env: any) => unknown>>;
        intersectedBy: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            by: Argument<Type.Container<(env: any) => unknown>>;
        }>>, Type.Container<(env: any) => unknown>>;
        within: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            target: Argument<Type.Container<(env: any) => unknown>>;
            'min-radius': Argument<Type.Value<number>>;
            'max-radius': Argument<Type.Value<number>>;
            'atom-radius': Argument<Type.Value<number>>;
            invert: Argument<Type.OneOf<boolean>>;
        }>>, Type.Container<(env: any) => unknown>>;
        isConnectedTo: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
            target: Argument<Type.Container<(env: any) => unknown>>;
            'bond-test': Argument<Type.OneOf<boolean>>;
            disjunct: Argument<Type.OneOf<boolean>>;
            invert: Argument<Type.OneOf<boolean>>;
        }>>, Type.Container<(env: any) => unknown>>;
    };
    combinator: {
        '@header': string;
        intersect: import("../symbol").MSymbol<Arguments<{
            [key: string]: (env: any) => unknown;
        }>, Type.Container<(env: any) => unknown>>;
        merge: import("../symbol").MSymbol<Arguments<{
            [key: string]: (env: any) => unknown;
        }>, Type.Container<(env: any) => unknown>>;
        distanceCluster: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            matrix: Argument<Type.Container<Core.Types.List<Core.Types.List<number>>>>;
            selections: Argument<Type.Container<Core.Types.List<(env: any) => unknown>>>;
        }>>, Type.Container<(env: any) => unknown>>;
    };
    atomSet: {
        '@header': string;
        atomCount: import("../symbol").MSymbol<Arguments<{}>, Type.Value<number>>;
        countQuery: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Container<(env: any) => unknown>>;
        }>>, Type.Value<number>>;
        reduce: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            initial: Argument<Type.Variable<any>>;
            value: Argument<Type.Variable<any>>;
        }>>, Type.Variable<any>>;
        propertySet: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
            0: Argument<Type.Variable<any>>;
        }>>, Type.Container<Core.Types.Set<any>>>;
    };
    atomProperty: {
        '@header': string;
        core: {
            '@header': string;
            elementSymbol: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            vdw: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            mass: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            atomicNumber: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            x: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            y: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            z: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            atomKey: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            bondCount: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
                flags: Argument<Type.Container<number>>;
            }>>, Type.Value<number>>;
            sourceIndex: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            operatorName: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            modelIndex: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            modelLabel: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
        };
        topology: {
            connectedComponentKey: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
        };
        macromolecular: {
            '@header': string;
            authResidueId: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            labelResidueId: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            residueKey: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            chainKey: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            entityKey: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            isHet: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            label_atom_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            label_alt_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            label_comp_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            label_asym_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            label_entity_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            label_seq_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            auth_atom_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            auth_comp_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            auth_asym_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            auth_seq_id: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            pdbx_PDB_ins_code: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            pdbx_formal_charge: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            occupancy: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            B_iso_or_equiv: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            entityType: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            entitySubtype: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            entityPrdId: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            entityDescription: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            objectPrimitive: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            secondaryStructureKey: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            secondaryStructureFlags: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            isModified: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            modifiedParentName: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            isNonStandard: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
            chemCompType: import("../symbol").MSymbol<Arguments<Arguments.PropTypes<{
                0: Argument<Type.Value<unknown>>;
            }>>, Type<any>>;
        };
    };
    bondProperty: {
        '@header': string;
        flags: import("../symbol").MSymbol<Arguments<{}>, Type<any>>;
        order: import("../symbol").MSymbol<Arguments<{}>, Type<any>>;
        length: import("../symbol").MSymbol<Arguments<{}>, Type<any>>;
        atomA: import("../symbol").MSymbol<Arguments<{}>, Type<any>>;
        atomB: import("../symbol").MSymbol<Arguments<{}>, Type<any>>;
    };
};
