"use strict";
/**
 * Copyright (c) 2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var argparse = tslib_1.__importStar(require("argparse"));
var util = tslib_1.__importStar(require("util"));
var fs = tslib_1.__importStar(require("fs"));
var structure_1 = require("../mol-model/structure");
var mmcif_1 = require("../mol-model/structure/export/mmcif");
var parser_1 = require("../mol-io/reader/gro/parser");
var gro_1 = require("../mol-model-formats/structure/gro");
var mol_task_1 = require("../mol-task");
require('util.promisify').shim();
var readFileAsync = util.promisify(fs.readFile);
var writeFileAsync = util.promisify(fs.writeFile);
function run(src, out) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var groResult, _a, traj, structure, _b, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    _a = parser_1.parseGRO;
                    return [4 /*yield*/, readFileAsync(src, 'utf8')];
                case 1: return [4 /*yield*/, _a.apply(void 0, [_d.sent()]).run()];
                case 2:
                    groResult = _d.sent();
                    if (groResult.isError)
                        throw new Error(groResult.message);
                    return [4 /*yield*/, gro_1.trajectoryFromGRO(groResult.result).run()];
                case 3:
                    traj = _d.sent();
                    _c = (_b = structure_1.Structure).ofModel;
                    return [4 /*yield*/, mol_task_1.Task.resolveInContext(traj.getFrameAtIndex(0))];
                case 4:
                    structure = _c.apply(_b, [_d.sent()]);
                    return [4 /*yield*/, writeFileAsync(out, mmcif_1.to_mmCIF(structure.label, structure, out.endsWith('.bcif')))];
                case 5:
                    _d.sent();
                    return [2 /*return*/];
            }
        });
    });
}
var parser = new argparse.ArgumentParser({
    addHelp: true,
    description: 'Convert GRO file to a CIF or BCIF file'
});
parser.addArgument(['src'], {
    help: 'Source GRO path'
});
parser.addArgument(['out'], {
    help: 'Output CIF/BCIF path'
});
var args = parser.parseArgs();
if (args) {
    run(args.src, args.out);
}
//# sourceMappingURL=test.js.map