/**
 * Copyright (c) 2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Vec2, Vec3 } from '../../mol-math/linear-algebra';
var ObjectHelper = /** @class */ (function () {
    function ObjectHelper(camera) {
        this.camera = camera;
        this.panMouseChange = Vec2();
        this.panObjUp = Vec3();
        this.panOffset = Vec3();
        this.eye = Vec3();
        this.panStart = Vec2();
        this.panEnd = Vec2();
    }
    ObjectHelper.prototype.setMouseOnScreen = function (out, pageX, pageY) {
        return Vec2.set(out, (pageX - this.camera.viewport.x) / this.camera.viewport.width, (pageY - this.camera.viewport.y) / this.camera.viewport.height);
    };
    ObjectHelper.prototype.getDirection = function (out, start, end) {
        this.setMouseOnScreen(this.panStart, start[0], start[1]);
        this.setMouseOnScreen(this.panEnd, end[0], end[1]);
        Vec2.sub(this.panMouseChange, Vec2.copy(this.panMouseChange, this.panEnd), this.panStart);
        Vec3.sub(this.eye, this.camera.position, this.camera.target);
        // TODO add support for custom z values/planes
        this.panMouseChange[0] *= (1 / this.camera.zoom) * this.camera.viewport.width;
        this.panMouseChange[1] *= (1 / this.camera.zoom) * this.camera.viewport.height;
        Vec3.cross(this.panOffset, Vec3.copy(this.panOffset, this.eye), this.camera.up);
        Vec3.setMagnitude(this.panOffset, this.panOffset, this.panMouseChange[0]);
        Vec3.setMagnitude(this.panObjUp, this.camera.up, this.panMouseChange[1]);
        Vec3.add(this.panOffset, this.panOffset, this.panObjUp);
        return Vec3.negate(out, this.panOffset);
    };
    return ObjectHelper;
}());
export { ObjectHelper };
//# sourceMappingURL=object-helper.js.map