/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Sebastian Bittrich <sebastian.bittrich@rcsb.org>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { __awaiter, __generator } from "tslib";
import { VdWLookup } from './common';
import { Vec3 } from '../../../../mol-math/linear-algebra';
import { StructureProperties, StructureElement } from '../../../../mol-model/structure/structure';
// TODO
// - iterate over units and elements
// - avoid using serial-element index whenever possible
// - calculate atomRadiusType only for invariant units
// - factor serialResidueIndex out
var updateChunk = 5000;
export function computeArea(runtime, ctx) {
    return __awaiter(this, void 0, void 0, function () {
        var atomRadius, i;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    atomRadius = ctx.atomRadiusType;
                    i = 0;
                    _a.label = 1;
                case 1:
                    if (!(i < atomRadius.length)) return [3 /*break*/, 5];
                    if (!runtime.shouldUpdate) return [3 /*break*/, 3];
                    return [4 /*yield*/, runtime.update({ message: 'Computing per residue surface accessibility...', current: i, max: atomRadius.length })];
                case 2:
                    _a.sent();
                    _a.label = 3;
                case 3:
                    computeRange(ctx, i, Math.min(i + updateChunk, atomRadius.length));
                    _a.label = 4;
                case 4:
                    i += updateChunk;
                    return [3 /*break*/, 1];
                case 5: return [2 /*return*/];
            }
        });
    });
}
var aPos = Vec3();
var bPos = Vec3();
var testPoint = Vec3();
function setLocation(l, structure, serialIndex) {
    l.structure = structure;
    l.unit = structure.units[structure.serialMapping.unitIndices[serialIndex]];
    l.element = structure.serialMapping.elementIndices[serialIndex];
    return l;
}
function computeRange(ctx, begin, end) {
    var structure = ctx.structure, atomRadiusType = ctx.atomRadiusType, serialResidueIndex = ctx.serialResidueIndex, area = ctx.area, spherePoints = ctx.spherePoints, scalingConstant = ctx.scalingConstant, maxLookupRadius = ctx.maxLookupRadius, probeSize = ctx.probeSize;
    var aLoc = StructureElement.Location.create(structure);
    var bLoc = StructureElement.Location.create(structure);
    var _a = StructureProperties.atom, x = _a.x, y = _a.y, z = _a.z;
    var lookup3d = structure.lookup3d, serialMapping = structure.serialMapping, unitIndexMap = structure.unitIndexMap;
    var cumulativeUnitElementCount = serialMapping.cumulativeUnitElementCount;
    for (var aI = begin; aI < end; ++aI) {
        var radius1 = VdWLookup[atomRadiusType[aI]];
        if (radius1 === VdWLookup[0])
            continue;
        setLocation(aLoc, structure, aI);
        var aX = x(aLoc);
        var aY = y(aLoc);
        var aZ = z(aLoc);
        // pre-filter by lookup3d (provides >10x speed-up compared to naive evaluation)
        var _b = lookup3d.find(aX, aY, aZ, maxLookupRadius), count = _b.count, units = _b.units, indices = _b.indices, squaredDistances = _b.squaredDistances;
        // collect neighbors for each atom
        var cutoff1 = probeSize + probeSize + radius1;
        var neighbors = []; // TODO reuse
        for (var iI = 0; iI < count; ++iI) {
            var bUnit = units[iI];
            StructureElement.Location.set(bLoc, ctx.structure, bUnit, bUnit.elements[indices[iI]]);
            var bI = cumulativeUnitElementCount[unitIndexMap.get(bUnit.id)] + indices[iI];
            var radius2 = VdWLookup[atomRadiusType[bI]];
            if (StructureElement.Location.areEqual(aLoc, bLoc) || radius2 === VdWLookup[0])
                continue;
            var cutoff2 = (cutoff1 + radius2) * (cutoff1 + radius2);
            if (squaredDistances[iI] < cutoff2) {
                neighbors[neighbors.length] = bI;
            }
        }
        // for all neighbors: test all sphere points
        Vec3.set(aPos, aX, aY, aZ);
        var scale = probeSize + radius1;
        var accessiblePointCount = 0;
        for (var sI = 0; sI < spherePoints.length; ++sI) {
            Vec3.scaleAndAdd(testPoint, aPos, spherePoints[sI], scale);
            var accessible = true;
            for (var _nI = 0; _nI < neighbors.length; ++_nI) {
                var nI = neighbors[_nI];
                setLocation(bLoc, structure, nI);
                Vec3.set(bPos, x(bLoc), y(bLoc), z(bLoc));
                var radius3 = VdWLookup[atomRadiusType[nI]];
                var cutoff3 = (radius3 + probeSize) * (radius3 + probeSize);
                if (Vec3.squaredDistance(testPoint, bPos) < cutoff3) {
                    accessible = false;
                    break;
                }
            }
            if (accessible)
                ++accessiblePointCount;
        }
        area[serialResidueIndex[aI]] += scalingConstant * accessiblePointCount * scale * scale;
    }
}
//# sourceMappingURL=area.js.map