/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { __awaiter, __extends, __generator } from "tslib";
import * as React from 'react';
import { PluginCommands } from '../../mol-plugin/commands';
import { PluginUIComponent } from '../base';
import { Button, ExpandGroup, ToggleButton } from '../controls/common';
import { CopySvg, CropFreeSvg, CropOrginalSvg, CropSvg, GetAppSvg } from '../controls/icons';
import { ParameterControls } from '../controls/parameters';
import { ScreenshotPreview } from '../controls/screenshot';
import { useBehavior } from '../hooks/use-behavior';
import { LocalStateSnapshotParams, StateExportImportControls } from '../state/snapshots';
var DownloadScreenshotControls = /** @class */ (function (_super) {
    __extends(DownloadScreenshotControls, _super);
    function DownloadScreenshotControls() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            showPreview: true,
            isDisabled: false
        };
        _this.download = function () {
            var _a;
            (_a = _this.plugin.helpers.viewportScreenshot) === null || _a === void 0 ? void 0 : _a.download();
            _this.props.close();
        };
        _this.copy = function () { return __awaiter(_this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, ((_a = this.plugin.helpers.viewportScreenshot) === null || _a === void 0 ? void 0 : _a.copyToClipboard())];
                    case 1:
                        _b.sent();
                        PluginCommands.Toast.Show(this.plugin, {
                            message: 'Copied to clipboard.',
                            title: 'Screenshot',
                            timeoutMs: 1500
                        });
                        return [2 /*return*/];
                }
            });
        }); };
        _this.copyImg = function () { return __awaiter(_this, void 0, void 0, function () {
            var src;
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, ((_a = this.plugin.helpers.viewportScreenshot) === null || _a === void 0 ? void 0 : _a.getImageDataUri())];
                    case 1:
                        src = _b.sent();
                        this.setState({ imageData: src });
                        return [2 /*return*/];
                }
            });
        }); };
        _this.open = function (e) {
            if (!e.target.files || !e.target.files[0])
                return;
            PluginCommands.State.Snapshots.OpenFile(_this.plugin, { file: e.target.files[0] });
        };
        return _this;
    }
    DownloadScreenshotControls.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.state.data.behaviors.isUpdating, function (v) {
            _this.setState({ isDisabled: v });
        });
    };
    DownloadScreenshotControls.prototype.componentWillUnmount = function () {
        this.setState({ imageData: void 0 });
    };
    DownloadScreenshotControls.prototype.render = function () {
        var _this = this;
        var _a;
        var hasClipboardApi = !!((_a = navigator.clipboard) === null || _a === void 0 ? void 0 : _a.write);
        return React.createElement("div", null,
            this.state.showPreview && React.createElement("div", { className: 'msp-image-preview' },
                React.createElement(ScreenshotPreview, { plugin: this.plugin }),
                React.createElement(CropControls, { plugin: this.plugin })),
            React.createElement("div", { className: 'msp-flex-row' },
                hasClipboardApi && React.createElement(Button, { icon: CopySvg, onClick: this.copy, disabled: this.state.isDisabled }, "Copy"),
                !hasClipboardApi && !this.state.imageData && React.createElement(Button, { icon: CopySvg, onClick: this.copyImg, disabled: this.state.isDisabled }, "Copy"),
                this.state.imageData && React.createElement(Button, { onClick: function () { return _this.setState({ imageData: void 0 }); }, disabled: this.state.isDisabled }, "Clear"),
                React.createElement(Button, { icon: GetAppSvg, onClick: this.download, disabled: this.state.isDisabled }, "Download")),
            this.state.imageData && React.createElement("div", { className: 'msp-row msp-copy-image-wrapper' },
                React.createElement("div", null, "Right click below + Copy Image"),
                React.createElement("img", { src: this.state.imageData, style: { width: '100%', height: 32, display: 'block' } })),
            React.createElement(ScreenshotParams, { plugin: this.plugin, isDisabled: this.state.isDisabled }),
            React.createElement(ExpandGroup, { header: 'State' },
                React.createElement(StateExportImportControls, { onAction: this.props.close }),
                React.createElement(ExpandGroup, { header: 'Save Options', initiallyExpanded: false, noOffset: true },
                    React.createElement(LocalStateSnapshotParams, null))));
    };
    return DownloadScreenshotControls;
}(PluginUIComponent));
export { DownloadScreenshotControls };
function ScreenshotParams(_a) {
    var plugin = _a.plugin, isDisabled = _a.isDisabled;
    var helper = plugin.helpers.viewportScreenshot;
    var values = useBehavior(helper.behaviors.values);
    return React.createElement(ParameterControls, { params: helper.params, values: values, onChangeValues: function (v) { return helper.behaviors.values.next(v); }, isDisabled: isDisabled });
}
function CropControls(_a) {
    var plugin = _a.plugin;
    var helper = plugin.helpers.viewportScreenshot;
    var cropParams = useBehavior(helper === null || helper === void 0 ? void 0 : helper.behaviors.cropParams);
    useBehavior(helper === null || helper === void 0 ? void 0 : helper.behaviors.relativeCrop);
    if (!helper)
        return null;
    return React.createElement("div", { style: { width: '100%', height: '24px', marginTop: '8px' } },
        React.createElement(ToggleButton, { icon: CropOrginalSvg, title: 'Auto-crop', inline: true, isSelected: cropParams.auto, style: { background: 'transparent', float: 'left', width: 'auto', height: '24px', lineHeight: '24px' }, toggle: function () { return helper.toggleAutocrop(); }, label: 'Auto-crop ' + (cropParams.auto ? 'On' : 'Off') }),
        !cropParams.auto && React.createElement(Button, { icon: CropSvg, title: 'Crop', style: { background: 'transparent', float: 'right', height: '24px', lineHeight: '24px', width: '24px', padding: '0' }, onClick: function () { return helper.autocrop(); } }),
        !cropParams.auto && !helper.isFullFrame && React.createElement(Button, { icon: CropFreeSvg, title: 'Reset Crop', style: { background: 'transparent', float: 'right', height: '24px', lineHeight: '24px', width: '24px', padding: '0' }, onClick: function () { return helper.resetCrop(); } }));
}
//# sourceMappingURL=screenshot.js.map