/**
 * Copyright (c) 2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { __awaiter, __generator } from "tslib";
import * as argparse from 'argparse';
import * as util from 'util';
import * as fs from 'fs';
import { Structure } from '../mol-model/structure';
import { to_mmCIF } from '../mol-model/structure/export/mmcif';
import { parseGRO } from '../mol-io/reader/gro/parser';
import { trajectoryFromGRO } from '../mol-model-formats/structure/gro';
import { Task } from '../mol-task';
require('util.promisify').shim();
var readFileAsync = util.promisify(fs.readFile);
var writeFileAsync = util.promisify(fs.writeFile);
function run(src, out) {
    return __awaiter(this, void 0, void 0, function () {
        var groResult, _a, traj, structure, _b, _c;
        return __generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    _a = parseGRO;
                    return [4 /*yield*/, readFileAsync(src, 'utf8')];
                case 1: return [4 /*yield*/, _a.apply(void 0, [_d.sent()]).run()];
                case 2:
                    groResult = _d.sent();
                    if (groResult.isError)
                        throw new Error(groResult.message);
                    return [4 /*yield*/, trajectoryFromGRO(groResult.result).run()];
                case 3:
                    traj = _d.sent();
                    _c = (_b = Structure).ofModel;
                    return [4 /*yield*/, Task.resolveInContext(traj.getFrameAtIndex(0))];
                case 4:
                    structure = _c.apply(_b, [_d.sent()]);
                    return [4 /*yield*/, writeFileAsync(out, to_mmCIF(structure.label, structure, out.endsWith('.bcif')))];
                case 5:
                    _d.sent();
                    return [2 /*return*/];
            }
        });
    });
}
var parser = new argparse.ArgumentParser({
    addHelp: true,
    description: 'Convert GRO file to a CIF or BCIF file'
});
parser.addArgument(['src'], {
    help: 'Source GRO path'
});
parser.addArgument(['out'], {
    help: 'Output CIF/BCIF path'
});
var args = parser.parseArgs();
if (args) {
    run(args.src, args.out);
}
//# sourceMappingURL=test.js.map