/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Áron Samuel Kovács <aron.kovacs@mail.muni.cz>
 */
declare const _default: "\nprecision highp float;\nprecision highp int;\nprecision highp sampler2D;\n\n// x: seed x coord\n// y: seed y coord\n// z: seed z coord\n// w: seed outline radius\nuniform sampler2D tOutlines;\nuniform vec2 uTexSize;\n\nuniform vec2 uStep;\nuniform float uOutlineViewRadius;\n\n#include common\n\nfloat atan2(float y, float x) {\n    return x == 0.0 ? sign(y) * PI / 2.0 : atan(y, x);\n}\n\nvoid getSamplingDxsDys(float angle, inout vec2 result[3]) {\n    angle = (angle / PI) * 180.0 + 180.0;\n    if (angle < 22.5 || angle > 337.5) {\n        result[0] = vec2(1.0, -1.0);\n        result[1] = vec2(1.0, 0.0);\n        result[2] = vec2(1.0, 1.0);\n    } else if (angle < 67.5) {\n        result[0] = vec2(1.0, 0.0);\n        result[1] = vec2(1.0, 1.0);\n        result[2] = vec2(0.0, 1.0);\n    } else if (angle < 112.5) {\n        result[0] = vec2(1.0, 1.0);\n        result[1] = vec2(0.0, 1.0);\n        result[2] = vec2(-1.0, 1.0);\n    } else if (angle < 157.5) {\n        result[0] = vec2(0.0, 1.0);\n        result[1] = vec2(-1.0, 1.0);\n        result[2] = vec2(-1.0, 0.0);\n    } else if (angle < 202.5) {\n        result[0] = vec2(-1.0, 1.0);\n        result[1] = vec2(-1.0, 0.0);\n        result[2] = vec2(-1.0, -1.0);\n    } else if (angle < 247.5) {\n        result[0] = vec2(-1.0, 0.0);\n        result[1] = vec2(-1.0, -1.0);\n        result[2] = vec2(0.0, -1.0);\n    } else if (angle < 292.5) {\n        result[0] = vec2(-1.0, -1.0);\n        result[1] = vec2(0.0, -1.0);\n        result[2] = vec2(1.0, -1.0);\n    } else /* if (angle < 337.5) */ {\n        result[0] = vec2(0.0, -1.0);\n        result[1] = vec2(1.0, -1.0);\n        result[2] = vec2(1.0, 0.0);\n    }\n}\n\nbool isInnerOutline(vec2 selfCoords, vec2 sampleCoords) {\n    vec2 dir = sampleCoords - selfCoords;\n    float angle = atan2(dir.y, dir.x);\n    vec2 sampleDxsDys[3];\n    getSamplingDxsDys(angle, sampleDxsDys);\n    return texture(tOutlines, sampleCoords + sampleDxsDys[0] / uTexSize).w < 0.0 ||\n           texture(tOutlines, sampleCoords + sampleDxsDys[1] / uTexSize).w < 0.0 ||\n           texture(tOutlines, sampleCoords + sampleDxsDys[2] / uTexSize).w < 0.0;\n}\n\nfloat outlineViewWidth(vec2 coordDiff) {\n    coordDiff = normalize(coordDiff);\n    float m = max(abs(coordDiff.x), abs(coordDiff.y));\n    coordDiff /= m;\n    return length(coordDiff) * uOutlineViewRadius;\n}\n\nvoid main(void) {\n    vec2 selfCoords = gl_FragCoord.xy / uTexSize;\n    vec4 result = texture(tOutlines, selfCoords);\n    float innerOutline = sign(result.w);\n\n    for (int dy = -1; dy <= 1; dy++) {\n        for (int dx = -1; dx <= 1; dx++) {\n            if (dx == 0 && dy == 0) {\n                continue;\n            }\n\n            vec2 sampleCoords = selfCoords + vec2(float(dx), float(dy)) * uStep;\n            if (sampleCoords.x < 0.0 || sampleCoords.x > 1.0 || sampleCoords.y < 0.0 || sampleCoords.y > 1.0) {\n                continue;\n            }\n\n            vec4 sampleValue = texture2D(tOutlines, sampleCoords);\n            vec2 sampleValueScreenCoords = sampleValue.xy;\n            if (sampleValueScreenCoords.x < 0.0) {\n                continue;                \n            }\n\n            vec2 coordDiff = (selfCoords.xy - sampleValueScreenCoords) * uTexSize;\n            float pixelDist = length(coordDiff);\n            if (pixelDist <= 3.6) {\n                float innerOutlineCorrection = isInnerOutline(selfCoords, sampleValueScreenCoords) ? 0.5 : 0.0;\n                pixelDist += innerOutlineCorrection;\n            }\n\n            if (sampleValue.z < result.z && pixelDist * abs(sampleValue.w) <= uOutlineViewRadius) {\n                result = sampleValue;\n            }\n        }\n    }\n\n    if (sign(result.w) != innerOutline) {\n        result.w *= -1.0;\n    }\n    gl_FragColor = result;\n}\n";
export default _default;
